#!/usr/bin/env python

"""
==============================================================================
SUPERPOSITION : Motif Finder
Authors: Osamu Maruyama (om@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
and
Daisuke Shinozaki (ma202035@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
Copyright (c) 2003 Osamu Maruyama and Daisuke Shinozaki
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE AUTHOR HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
==============================================================================
"""

import sys,re,string,time,random,math
import revcomp

aRevComp = revcomp.RevComp()

############### CLASS SCORELIST #####

class ScoreList:


    def __init__(self,NumTop,Maxlen,Minlen):

        self.ScoreList = []
        self.NumTop = NumTop
        self.Maxlen = Maxlen
        self.Minlen = Minlen


    def return_order(self):
        
        if self.Maxlen == self.Minlen:
            return [self.Maxlen]
        
        OriginalOrder = []
        Order = []
        OrderMap = {}
        for i in range(self.Minlen,self.Maxlen+1):
            OriginalOrder.append(i)

        scorelist = self.ScoreList
        for i in range(1,len(scorelist)+1):
            x = len(scorelist[-i][3])
            if not OrderMap.has_key(x):
                Order.append(x)
                OrderMap[x] = 0
                OriginalOrder.remove(x)
            if len(OriginalOrder) == 1:
                Order = Order + OriginalOrder
                break

        return Order
            
        

    def push_and_sort(self,PatternData):
        
        self.ScoreList.append(PatternData)
        self.ScoreList.sort()
        if len(self.ScoreList) > self.NumTop:
            self.ScoreList = self.ScoreList[1:]


    def get_worst_score(self):

        return self.ScoreList[0][0]


    def get_all(self):

        return self.ScoreList
        

    def judge(self,value):

        if value > self.get_worst_score():
            return 1
        else:
            return 0
        


############### CLASS BUFFERLIST #####

class BufferList:


    def __init__(self):

        self.BufList = []

        
    def push_and_sort(self,PatternData):
        
        self.BufList.append(PatternData)
        self.BufList.sort()    


    def get_at(self,num):

        return self.BufList[-num]


    def remove(self,num):

        if num != 0:
            del self.BufList[:-num+1]
        else:
            self.BufList = []
        

    def concat(self,another):

        self.BufList = self.BufList + another.BufList
        self.BufList.sort()        


    def pop(self):
        
        x = self.BufList[-1]
        del self.BufList[-1]
        return x


    def initialize(self):

        self.BufList = []


    def push_BufList(self,length,worst_score,BUFLIST):

        for eachPat in self.BufList:
            if len(eachPat[2]) == length:
                if eachPat[0] > worst_score:
                    BUFLIST.push_and_sort(eachPat)
                self.BufList.remove(eachPat)
                
        return BUFLIST



########## CLASS SCOREFUNCTION #####
    
class ScoreFunction:


    def __init__(self,lenP,lenN):

        self.sizeN = lenN
        self.sizeP = lenP


    def upperbound(self,tp,fp):

        UpperBound = cal_score_tp_fp_multiple(self.sizeP,self.sizeN,self.sizeP,fp)
        return UpperBound
    

    def score(self,tp,fp):

        RetVal = cal_score_tp_fp_multiple(self.sizeP,self.sizeN,tp,fp)
        return RetVal
        


############### CLASS PATTERNMAP #####

class PatternMap:


    def __init__(self):

        self.PatMap = {}
        
    
    def set_MinPattern(self,Pattern):
 
        min_key = min(Pattern,aRevComp.make(Pattern))
        self.PatMap[min_key] = 0
        return min_key


    def not_has_key(self,Pattern):

        key = min(Pattern,aRevComp.make(Pattern))
        if self.PatMap.has_key(key):
            return 0
        else:
            return 1


###############Score Function #
    

def cal_threFP_RatioSum(lenN,threSCORE):

    RetFP = lenN*(2-threSCORE)
    return RetFP


def cal_score_W_Ratio(p,n,tp,fp):

    left = float(tp)/float(p)
    right = float(n-fp)/float(n)
    RetVal = left + right
    return RetVal


def cal_score_W_Ratio_square(p,n,tp,fp):

    left = float(tp)/float(p)
    right = float(n-fp)/float(n)
    left2 = pow(left,2)
    right2 = pow(right,2)    
    RetVal = left2 + right2
    return RetVal


def cal_score_W_Ratio_sqrt(p,n,tp,fp):

    left = float(tp)/float(p)
    right = float(n-fp)/float(n)
    left2 = math.sqrt(left)
    right2 = math.sqrt(right)    
    RetVal = left2 + right2
    return RetVal


def cal_score_tp_plus_tn(p,n,tp,fp):
    
    return float(tp)+float(n)-float(fp)


def cal_score_sqrttp_squarefp(p,n,tp,fp):

    left = float(tp)/float(p)
    right = float(n-fp)/float(n)
    left2 = math.sqrt(left)
    right2 = float(pow(right,2.0))    
    RetVal = left2 + right2
    return RetVal


def cal_score_Mtp_Ntn(posi,nega,tp,fp,m,n):

    left = float(tp)/float(p)
    right = float(n-fp)/float(n)
    left2 = pow(left,float(m))
    right2 = pow(right,float(n))
    RetVal = left2 + right2
    return RetVal

def cal_score_tp_fp_multiple(p,n,tp,fp):
     
    left = float(tp)/float(p)
    right = float(n-fp)/float(n)
    left2 = math.sqrt(left)
    right2 = float(pow(right,2.0))
    RetVal = left2 * right2
    return RetVal
                             
