#!/usr/bin/env python

"""
==============================================================================
SUPERPOSITION : Motif Finder
Authors: Osamu Maruyama (om@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
and
Daisuke Shinozaki (ma202035@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
Copyright (c) 2003 Osamu Maruyama and Daisuke Shinozaki
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE AUTHOR HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
==============================================================================
"""

import sys,re,string
import unionIUPACany,Spp_Class,revcomp,Spp_extract,Spp_util,SaveFile

aRevComp = revcomp.RevComp()
Dcheck = re.compile('[MKRYSWBDHVN]')

###############
#    MAIN     #
###############

def Superpose(SCORELIST,BUFLIST1,BUFLIST2,ExSeq,SCOREFUNCTION,_logSave,UpperDgen,Spacer,LenLR):
    
    PATMAP = Spp_Class.PatternMap()
    counter = 0
    
    while len(BUFLIST1.BufList) > 1:
        Pattern1 = BUFLIST1.pop()
        print
        print 'Superposition with ',Pattern1[2]
        print '      upperbound : ',Pattern1[0]
        lenBUF1 = len(BUFLIST1.BufList)
        print '   |sorted list| : ',str(lenBUF1)
        print '          maxVal : ',SCORELIST.get_worst_score()
        Bcounter = counter
        if SCORELIST.judge(Pattern1[0]) == 0:
            break
        if degeneratively_value(Pattern1[2]) <= UpperDgen/2:  
            for i in range(1,lenBUF1+1):
                Pattern2 = BUFLIST1.get_at(i)   
                if SCORELIST.judge(Pattern2[0]) == 0:
                    BUFLIST1.remove(i)   
                    break
                
                NewPat1 = make_NewPat(Pattern1[2],Pattern2[2],UpperDgen) 
                NewPat2 = make_NewPat(Pattern1[2],aRevComp.make(Pattern2[2]),UpperDgen)
                if Spacer:
                    if LenLR % 2 != 0:
                        NewPats = [NewPat1]
                    else:
                        NewPats = [NewPat1,NewPat2]             
                else:
                    NewPats = [NewPat1,NewPat2]             
                
                for eachPat in NewPats:
                    if PATMAP.not_has_key(eachPat):
                        if degeneratively_value(eachPat) <= UpperDgen:
                            minPat = PATMAP.set_MinPattern(eachPat)
                            counter = counter + 1
                            Pat4data = calculate_tp_fp_score(minPat,ExSeq,SCOREFUNCTION)
                            SCORELIST,BUFLIST2 = Spp_util.subroutine(Pat4data,SCORELIST,SCOREFUNCTION,BUFLIST2)

        
        BUFLIST1.concat(BUFLIST2)
        Acounter = counter
        lenBUF1After = len(BUFLIST1.BufList)
        SaveFile.eachlog(lenBUF1,lenBUF1After,Bcounter,Acounter,_logSave,Pattern1,SCORELIST)
        BUFLIST2.initialize()

    return SCORELIST
    

############### sub 0 #

def degeneratively_value(Pattern):

    Value = 1
    Valtab = {'A':1,'C':1,'T':1,'G':1,
              'M':2,'K':2,'S':2,'W':2,'R':2,'Y':2,
              'B':3,'D':3,'H':3,'V':3,
              'N':4
              }
    for i in range(len(Pattern)):
        Value = Value * Valtab[Pattern[i]]

    return Value



############### sub 1 #
            
def make_NewPat(Pat1,Pat2,UpperDgen):

    currentDgen = 1 
    NewPat = ''
    for i in range(len(Pat1)):
        if Pat1[i] != Pat2[i]:
            charaD = unionIUPACany.change_degenerate(Pat1[i],Pat2[i])
            NewPat = NewPat + charaD
        else:
            NewPat = NewPat + Pat1[i]
    return NewPat
        
   
############### sub 2 #

def calculate_tp_fp_score(NewPat,ExSeq,SCOREFUNCTION):
    
    
    re_NewPat = NewPat_to_re_NewPat(NewPat) # process 1
    re_NewPatRC = NewPat_to_re_NewPat(aRevComp.make(NewPat))

    re_NewPat_search = re_NewPat + '|' + re_NewPatRC 
    
    Re_NewPat = re.compile(re_NewPat_search)

    tp = 0
    fp = 0

    for each in ExSeq.keys():
        
        if each == 'P':
            for eachSeq in ExSeq[each]:
                B = re.search(Re_NewPat,eachSeq)
                if B:
                    tp = tp + 1
        else:
            for eachSeq in ExSeq[each]:
                B = re.search(Re_NewPat,eachSeq)
                if B:
                    fp = fp + 1

    aScore = SCOREFUNCTION.score(tp,fp)
    Pat4data = (aScore,tp,fp,NewPat)

    return Pat4data
                          
        
############### sub 2-1 #

def NewPat_to_re_NewPat(NewPat):
    
    
    ctab = {'A':'A','C':'C','G':'G','T':'T',
            'M':'[AC]','K':'[GT]','R':'[AG]',
            'Y':'[TC]','W':'[AT]','S':'[CG]',
            'B':'[CGT]','D':'[AGT]',
            'H':'[ATC]','V':'[ACG]',
            'N':'[ACGT]'
            }

    RetPat = ''
    for i in range(len(NewPat)):
        RetPat = RetPat + ctab[NewPat[i]]
    
    return RetPat
