#!/usr/bin/env python

"""
==============================================================================
SUPERPOSITION : Motif Finder
Authors: Osamu Maruyama (om@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
and
Daisuke Shinozaki (ma202035@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
Copyright (c) 2003 Osamu Maruyama and Daisuke Shinozaki
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE AUTHOR HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
==============================================================================
"""

import sys,re,time
import util,revcomp,Spp_Class,unitool,Spp_util,Spp_Superpose
import SaveFile

aTab = { 'a' : 'ARMW',
         'c' : 'CYMS',
         'g' : 'GRKS',
         't' : 'TYKW',
         'A' : 'ARMW',
         'C' : 'CYMS',
         'G' : 'GRKS',
         'T' : 'TYKW'
         }


############### MAIN #

def main_routine(Output,ExSeq,LenLR,NumTop,UpperDgen,Spacer):

    Output_log = Output.name + '.lg'
    _logSave = Output_log
    logSave = open(_logSave,'w+')
    logSave.close()
    
    OccurrenceCountTree = {}

    if Spacer:
        OccurrenceCountTree = __extract_with_spacer(ExSeq,LenLR,OccurrenceCountTree,Spacer)
    else:
        OccurrenceCountTree = __extract(ExSeq,LenLR,OccurrenceCountTree)

    PosExNum = len(ExSeq['P'])
    NegExNum = len(ExSeq['N'])

    SCORELIST = Spp_Class.ScoreList(NumTop,LenLR,LenLR)
    SCOREFUNCTION = Spp_Class.ScoreFunction(PosExNum,NegExNum)
    BUFLIST1 = Spp_Class.BufferList()
    BUFLIST2 = Spp_Class.BufferList()

    lenOCT = len(OccurrenceCountTree)
    
    for aSingle in OccurrenceCountTree.keys():

        Pat4data = __calculate_tp_fp_score(OccurrenceCountTree,aSingle,SCOREFUNCTION)  
        SCORELIST,BUFLIST1 = Spp_util.subroutine(Pat4data,SCORELIST,SCOREFUNCTION,BUFLIST1)

    scorelist = SCORELIST.get_all()
    SaveFile.save_extract_data(scorelist,Output)
        
    del OccurrenceCountTree

    wScore = SCORELIST.get_worst_score()

    logSave = open(_logSave,'a')
    logSave.write('*** SuperPosing length ' + str(LenLR) + ' Patterns ***')
    logSave.write('\n')
    logSave.write('Start: ' +  time.ctime(time.time()) + '\n')
    logSave.write('extracted pattern: ' +str(lenOCT)+ '\n')
    logSave.write('*************************************')
    logSave.write('\n')
    logSave.close()

    if Spacer:
        UpperDgen = UpperDgen * pow(4,Spacer)
        
    SCORELIST = Spp_Superpose.Superpose(SCORELIST,BUFLIST1,BUFLIST2,ExSeq,SCOREFUNCTION,_logSave,UpperDgen,Spacer,LenLR)

    logSave = open(_logSave,'a')
    logSave.write('\n')
    logSave.write('*** SuperPosing length ' + str(LenLR) + ' Finished ***')
    logSave.write('\n')
    logSave.write('End: ' +  time.ctime(time.time()) + '\n')
    logSave.write('*************************************')
    logSave.write('\n')
    logSave.write('\n')
    logSave.write('\n')
    logSave.close()
    
    BUFLIST1.initialize()


    SaveFile.save_Scorefiledata(SCORELIST.ScoreList,NumTop,Output)

    
    Output.write('# end: ' + time.ctime(time.time()) + '\n')
    Output.close()
    logSave.close()
    print 'end'
    sys.exit()

################ sub 1 #

def __extract(ExSeq,LenLR,OccurrenceCountTree):

    aRevComp = revcomp.RevComp()

    for Ex in ExSeq.keys():
        for SeqID in range(len(ExSeq[Ex])):
            T = ExSeq[Ex][SeqID] 
            for aPos in range(len(T)-LenLR+1):
                Key1 = T[aPos:aPos+LenLR] 
                Key2 = aRevComp.make(Key1)
                theKey = min(Key1,Key2) 

                aGain = 1
                if Key1 == Key2:
                    aGain = 2

                if Ex == 'P':
                    if not OccurrenceCountTree.has_key(theKey):
                        OccurrenceCountTree[theKey] = {}
                        for each in ExSeq.keys(): # = ['P','N']
                            OccurrenceCountTree[theKey][each] = [0] * len(ExSeq[each])
                                        
                if OccurrenceCountTree.has_key(theKey):
                    OccurrenceCountTree[theKey][Ex][SeqID] = OccurrenceCountTree[theKey][Ex][SeqID] + aGain

    return OccurrenceCountTree

################ sub 1' #

def __extract_with_spacer(ExSeq,LenLR,OccurrenceCountTree,Spacer):

    LenL = LenLR/2
    LenR = LenLR - LenL
    
    aRevComp = revcomp.RevComp()

    for Ex in ExSeq.keys():
        for SeqID in range(len(ExSeq[Ex])):
            T = ExSeq[Ex][SeqID] 
            for aPos in range(len(T)-LenLR-Spacer+1):
                PartL = T[aPos:aPos+LenL]
                PartR = T[aPos+LenL+Spacer:aPos+LenL+Spacer+LenR]
                Key1 = PartL + 'N'*Spacer + PartR
                if LenL == LenR:
                    Key2 = aRevComp.make(PartR) + 'N'*Spacer + aRevComp.make(PartL)
                    theKey = min(Key1,Key2)
                    keyset = [theKey]

                else:
                    rcPartR = aRevComp.make(T[aPos:aPos+LenR])
                    rcPartL = aRevComp.make(T[aPos+LenR+Spacer:aPos+LenR+Spacer+LenL])
                    Key2 = rcPartL + 'N'*Spacer + rcPartR
                    keyset = (Key1,Key2)
                                        
                aGain = 1
                if len(keyset) == 1:
                    if Key1 == Key2:
                        aGain = 2

                if Ex == 'P':
                    for eachKey in keyset:
                        if not OccurrenceCountTree.has_key(eachKey):
                            OccurrenceCountTree[eachKey] = {}
                            for each in ExSeq.keys(): # = ['P','N']
                                OccurrenceCountTree[eachKey][each] = [0] * len(ExSeq[each])

                for eachKey in keyset:
                    if OccurrenceCountTree.has_key(eachKey):
                        OccurrenceCountTree[eachKey][Ex][SeqID] = OccurrenceCountTree[eachKey][Ex][SeqID] + aGain

    return OccurrenceCountTree


############### sub 2 #

def __calculate_tp_fp_score(OccurrenceCountTree,aSingle,SCOREFUNCTION):


    PosList = OccurrenceCountTree[aSingle]['P']
    NegList = OccurrenceCountTree[aSingle]['N']
    
    tp = 0
    for x in PosList:
        if x >= 1:
            tp = tp + 1

    fp = 0
    for x in NegList:
        if x >= 1:
            fp = fp + 1
                
    score = SCOREFUNCTION.score(tp,fp) 
    Pat4data = (score,tp,fp,aSingle)

    return Pat4data

