#!/usr/bin/env python

"""
==============================================================================
SUPERPOSITION : Motif Finder
Authors: Osamu Maruyama (om@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
and
Daisuke Shinozaki (ma202035@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
Copyright (c) 2003 Osamu Maruyama and Daisuke Shinozaki
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE AUTHOR HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
==============================================================================
"""

import sys,string,re
import genomelib,config

class GenBankFile:

    def __init__(self, file_name):
        self._scan(file_name)

    def get(self):
        "fadfalflassdf"
        return self.before_features, self.features, self.sequence

    def _scan(self, file_name):
        self.input = open(file_name)

        self._scan_before_features()
        self._scan_features()
        self._scan_sequence()
        
    def _scan_before_features(self):
        self.before_features = [] 
        self.line = self.input.readline()
        print string.strip(self.line)
        while 1:
            if "FEATURES" != self.line[:8]:
                self.before_features.append(string.strip(self.line))
                self.line = self.input.readline()
            else:
                break
                
    def _scan_features(self):
        self.features = []
        if "FEATURES" == self.line[:8]:
            self.line = self.input.readline()
        else:
            print "Error:_scan_features()"
            sys.exit(1)
        while "BASE" != self.line[:4]:
            self._scan_one_feature()

    def _scan_one_feature(self):
        if " " == self.line[5]:
            print "Error:_scan_one_feature()"
            sys.exit(1)
        else:
            buf = [string.strip(self.line)]
            self.line = self.input.readline()
            while ((" " == self.line[5]) and ("BASE" != self.line[:4])):
                buf.append(string.strip(self.line))
                self.line = self.input.readline()
        self._postprocess_feature(buf)

    def _postprocess_feature(self, buf):
        # The first line contains a feature name and (possibly, part of)
        # a location.
        first_line = string.split(string.strip(buf[0]))
        if 2 != len(first_line):
            print "Error: GenBankFile: _postprocess_feature(self, buf)"
            exit(1)
        folder = [[first_line[0]], [first_line[1]]]
        
        for p in range(1, len(buf)):
            if "/" == buf[p][0]:
                folder.append([buf[p]])
            else:
                folder[len(folder) - 1].append(buf[p]) 

        for i in range(len(folder)):
            folder[i] = string.join(folder[i], "")
            
        self.features.append(folder)
        

    def _scan_sequence(self):
        self.sequence = "" 
        self.line = self.input.readline() 
        self.line = self.input.readline() 

        buf = [] 
        while ("//" != self.line[0:2]):
            # s = string.split(string.strip(self.line))
            # buf.extend(s[1:])

            buf.extend(string.split(string.strip(self.line))[1:])
            self.line = self.input.readline()
        self.sequence = string.join(buf, "")


class UpstreamRegion:
    class _Attribute:
        def __init__(self):
            self.sequence = ""
            self.GENE = None # gene name, namely common name.
            # self.complement = ""
        def get_sequence(self):
            return self.sequence
        def get_complement(self):
            # return self.complement
            return genomelib.complement(self.sequence)
        def get_GENE_name(self):
            return self.GENE

    def __init__(self):
        # key: gene and ORF name
        # value: Attribute object
        self.map_ORF_to_Attribute = {}
        self.map_GENE_to_ORF = {}

    def make(self,organism,DB_DIR,relative_region):
        SRC_DIR,file_names = self.__organism(DB_DIR)
        for prefix in file_names:
            b_f, features, sequence = GenBankFile(SRC_DIR + prefix + ".gbk").get()
            self.__engine(features,sequence,relative_region)

    def __organism(self,DB_DIR):
        SRC_DIR = None
        file_names = None

        if DB_DIR:
            SRC_DIR = DB_DIR + "/"
            file_names = config.DB_FILE_NAMES

            # ["NC_001133", "NC_001134", "NC_001135", "NC_001136", "NC_001137", "NC_001138", "NC_001139", "NC_001140", "NC_001141", "NC_001142", "NC_001143", "NC_001144", "NC_001145", "NC_001146", "NC_001147", "NC_001148"]

        return SRC_DIR,file_names



    def open(self,filename):
        input = open(filename)
        line = input.readline()

        while line[:2] != "//":

            """name part"""
            names = string.split(line)
            ORF_name = names[1]
            GENE_name = None
            len_names = len(names)
            if len_names == 3:
                GENE_name = names[2]
            if len_names < 2 or len_names > 3:
                print "Note:", names

            """sequence part"""
            splited_sequence = []
            line = input.readline()
            while line[0] != '>' and line[:2] != "//":
                line = string.strip(line)
                splited_sequence.append(line)
                line = input.readline()
            sequence = string.join(splited_sequence, "")

            """stuffing"""
            att = self._Attribute()
            att.sequence = sequence
            self.map_ORF_to_Attribute[ORF_name] = att 
            if GENE_name:
                # self.map_ORF_to_Attribute[GENE_name] = att 
                att.GENE = GENE_name
                self.map_GENE_to_ORF[GENE_name] = ORF_name


    def save(self,filename,OneLineLength = 50):
        output = open(filename,'w')

        # re.compile("Y[A-P][LR]\d{3,3}[WC](?:-[A-Z])*")
        # PAT = re.compile("Y[A-P][LR]\d{3,3}[WC]*")
        
        for name in self.map_ORF_to_Attribute.keys():
            output.write(self.make_fasta_format(name,OneLineLength))
                
        output.write("//\n")
        output.close()

    def make_fasta_format(self,name,OneLineLength):
        RetVal = ""
        value = self.map_ORF_to_Attribute[name]
        RetVal = RetVal + '> ' + name
        if value.GENE:
            RetVal = RetVal + ' ' + value.GENE
        RetVal = RetVal + '\n'

        # replacable with genomelib.get_fasta_format
        for i in range(0,len(value.sequence),OneLineLength):
            RetVal = RetVal + value.sequence[i:i+OneLineLength] + '\n'
        return RetVal


    def get_ORF_name_from_GENE_name(self,GENE):
        return self.map_GENE_to_ORF[GENE]
    def get_GENE_name_from_ORF_name(self,ORF):
        return self.map_ORF_to_Attribute[ORF].GENE


    def get_GENE_names(self):
        return self.map_GENE_to_ORF.keys()

    def get_all_sequences(self):
        aRetVal = {}
        for each in self.map_ORF_to_Attribute.values():
            aRetVal[each.get_sequence()] = 0
        return aRetVal.keys()

    def __len__(self):
        return len(self.map_ORF_to_Attribute)
            
    def __getitem__(self,key):
        if self.map_ORF_to_Attribute.has_key(key):
            return self.map_ORF_to_Attribute[key]
        elif self.map_GENE_to_ORF.has_key(key):
            GENE = self.map_GENE_to_ORF[key]
            return self.map_ORF_to_Attribute[GENE]
        else:
            # print "Warning: class UpstreamRegion:__getitem__(self,key):No such a key:", key
            return None
            

    def has_key(self, key):
        return self.map_ORF_to_Attribute.has_key(key)

    def keys(self):
        return self.map_ORF_to_Attribute.keys()

    def items(self):
        pass
        # return self.map_ORF_to_Attribute.items()

                               

    def _find_qualifier(self, list, qualifier, start = 0):
        """Return the value of a specified qualifier. 
        Suppose the form of qualifier=value.
        """
        for p in range(start, len(list)):
            if qualifier == list[p][:len(qualifier)]:
                return p
        return -1

    def __engine(self,features,sequence,relative_region):
        location_parser = re.compile("\d+")
        for f in features:

            SLASH_GENE = None
            SLASH_NOTE = None
            
            if "gene" != f[0]: # Collect only the features of "gene".
                continue
            att = self._Attribute() 

            location_original = f[1]
            base_region = location_parser.findall(location_original)
            
            for p in range(len(base_region)):
                base_region[p] = int(base_region[p])

            complement_pos = string.find(location_original, "complement")
            if -1 == complement_pos:
                is_complement = None 
                ur = genomelib.upstream_region(is_complement,base_region,relative_region)
                
                """
                When ur[0]-1 is a negative integer, 
                sequence[ur[0]-1 : ur[1]] becomes an empty string (''). 
                It is the reason why the first element of sequence is
                max(0,ur[0]-1).
                """
                att.sequence = sequence[max(0,ur[0]-1) : ur[1]]
                ### att.complement = genomelib.complement(att.sequence)
            elif 0 == complement_pos:
                
                is_complement = 1
                ur = genomelib.upstream_region(is_complement,base_region,relative_region)
                
                ### att.complement = sequence[ur[0]-1 : ur[1]]
                ### att.sequence = genomelib.complement(att.complement)
                att.sequence = genomelib.complement(sequence[ur[0]-1 : ur[1]])
            else:
                
                print f
                print "complement is not a prefix of location"
                sys.exit(0)


            # The extracted sequences are capitalized.
            att.sequence = string.upper(att.sequence)
            ### att.complement = string.upper(att.complement)

            p = self._find_qualifier(f, "/gene")
            if p >= 0:
                SLASH_GENE = f[p][7:-1]

            p = self._find_qualifier(f, "/note")
            if p >= 0:
                if ((-1 == string.find(f[p][7:], ";")) and
                    (-1 == string.find(f[p][7:], " "))):

                    SLASH_NOTE = f[p][7:-1]

            if SLASH_NOTE:
                """SLASH_NOTE is ORF name"""
                """SHASH_GENE is GENE name"""
                self.map_ORF_to_Attribute[SLASH_NOTE] = att
                att.GENE = SLASH_GENE
                self.map_GENE_to_ORF[att.GENE] = SLASH_NOTE
            else:
                self.map_ORF_to_Attribute[SLASH_GENE] = att

def test_open():
    g = UpstreamRegion()
    g.open("removeme")
    return g
    
def test_save(g):
    g.save("removeme")

def test_load():
    import config
    g = UpstreamRegion()
    organism = "S_cerevisiae"
    g.make(organism,config.DB_DIR,[-10,-1])
    return g

def basic_test(g):
    assert("ACGCTAACAA" == g["YAL069W"].sequence)
    assert("TTGTTAGCGT" == g["YAL069W"].get_complement())

    assert("AACAAATACA" == g["YAL068C"].sequence)
    assert("TGTATTTGTT" == g["YAL068C"].get_complement())

    assert(g["YAL068C"])
    assert(g["GAL2"])
    assert(None == g["xxx"])

    assert('YNR035C' == g.get_ORF_name_from_GENE_name('ARC35'))
    assert('ARC35' == g.get_GENE_name_from_ORF_name('YNR035C'))

    return g

def test_save_and_open():
    import config
    g = UpstreamRegion()
    organism = "S_cerevisiae"
    g.make(organism,config.DB_DIR,[-800,-1])
    g.save("removeme")

    t = UpstreamRegion()
    t.open("removeme")

    print len(g), len(t)
    print len(g.map_GENE_to_ORF), len(t.map_GENE_to_ORF)
    
    assert(len(g) == len(t))
    assert(len(g.map_GENE_to_ORF) == len(t.map_GENE_to_ORF))

    for ORF in g.keys():
        assert(g[ORF].get_sequence() == t[ORF].get_sequence())
        assert(g[ORF].GENE == t[ORF].GENE)
    
    

def test():
    # g = test_load()

    print "start open"
    g = test_open()

    # test_save(g)

    # print "start basic_test"
    # basic_test(g)

    print "start save"
    g.save("removeme2")

    print "end everything"


if __name__ == "__main__":
    test()

    # test_save_and_open()



    
    
