#!/usr/bin/python

"""
==============================================================================
SUPERPOSITION : Motif Finder
Authors: Osamu Maruyama (om@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
and
Daisuke Shinozaki (ma202035@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
Copyright (c) 2003 Osamu Maruyama and Daisuke Shinozaki
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE AUTHOR HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
==============================================================================
"""

import string
import sys
import re

import util

tab_complement = {"a" : "t",
                  "t" : "a",
                  "c" : "g",
                  "g" : "c",
                  "A" : "T",
                  "T" : "A",
                  "C" : "G",
                  "G" : "C"
                  }


def get_fasta_format(seqence,length = 50):
    RetVal = ""
    for i in range(0,len(sequence),length):
        RetVal = RetVal + sequence[i:i+legth] + '\n'
    return RetVal

def count_freq(aStr, aChars):
    aSum = 0
    for aChar in aChars:
        aSum = aSum + string.count(aStr, aChar)
    return float(aSum)/len(aStr)


def gene_name_matcher(organism):
    if "S_cerevisiae" == organism:
        return re.compile("Y[A-P][LR]\d{3,3}[WC](?:-[A-Z])*")
        
def complement(sequence):
    buf = []
    N = len(sequence)
    for p in range(N):
        buf.append(tab_complement[sequence[N - p - 1]])
    return string.join(buf, "")


def upstream_region(is_complement, base_region, relative_region):
    absolute_region = [1, 1]
    if not is_complement:
        absolute_region[0] = base_region[0] + relative_region[0]
        absolute_region[1] = base_region[0] + relative_region[1]

        if relative_region[0] > 0:
            absolute_region[0] = absolute_region[0] - 1

        if relative_region[1] > 0:
            absolute_region[1] = absolute_region[1] - 1
    else:
        absolute_region[0] = base_region[-1] - relative_region[1]
        absolute_region[1] = base_region[-1] - relative_region[0]

        if relative_region[1] > 0:
            absolute_region[0] = absolute_region[0] + 1
        if relative_region[0] > 0:
            absolute_region[1] = absolute_region[1] + 1
    return absolute_region


if __name__ == "__main__":

    assert("acGtT" == complement("AaCgt"))

    s = DNASequence("ATCG")
    assert(s.get() == "ATCG")
    assert(s.get_complement() == "CGAT")
    assert(s.get_complement(1,4) == "CGAT")
    assert(s.get_complement(2,3) == "GA")
    assert(s.get_complement(0,1) == "T")
    assert(s.get_complement(4,4) == "C")
    assert(s.get_complement(4,5) == "C")
    assert(s.get_complement(1,2) == "AT")
    print s

    is_complement = None
    assert([7,9] == upstream_region(is_complement, [10, 20], [-3,-1]))
    assert([9,11] == upstream_region(is_complement, [10, 20], [-1,2]))
    assert([10,12] == upstream_region(is_complement, [10, 20], [1,3]))

    is_complement = 1
    assert([21,23] == upstream_region(is_complement, [10, 20], [-3,-1]))
    assert([19,21] == upstream_region(is_complement, [10, 20], [-1,2]))
    assert([18,20] == upstream_region(is_complement, [10, 20], [1,3]))
    

    


