#!/usr/bin/env python

"""
==============================================================================
SUPERPOSITION : Motif Finder
Authors: Osamu Maruyama (om@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
and
Daisuke Shinozaki (ma202035@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
Copyright (c) 2003 Osamu Maruyama and Daisuke Shinozaki
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE AUTHOR HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
==============================================================================
"""

import string, random

ALPHABET = ['A','T','C','G']
INIT_VAL = 500000000

def generate(k):
    _store = ALPHABET[:]
    for _length in range(k-1):
        _storenew = []
        for _string in _store:
            for _char in ALPHABET:
                _storenew.append(_string + _char)
        _store = _storenew 
    return _store

class MarkovModel:
    def __init__(self):
        self.order = ['A','T','C','G']

    def initialize(self,k,sequences):
        print 'MarkovModel'

        self.map = {}

        # initializing self.map.
        for _s in generate(k):
            self.map[_s] = {}  
            for _c in ALPHABET:
                self.map[_s][_c] = 0

        print 'Reading backgroud sequences...'
        for _seq in sequences:
            for _pos in range(len(_seq)-k):
                self.map[_seq[_pos:_pos+k]][_seq[_pos+k]] = self.map[_seq[_pos:_pos+k]][_seq[_pos+k]] + 1

        print 'Postprocessing...'
        for (_kmer,_local_map) in self.map.items():
            _sum = 0
            for _val in _local_map.values():
                _sum = _sum + _val
            if _sum == 0:
                for _c in ALPHABET: 
                    self.map[_kmer][_c] = INIT_VAL


    def save(self,filename):
        fout = open(filename,'w')
        fout.write('#' + str(self.order) + '\n')
        for kmer in self.map.keys():
            fout.write(kmer + '\n')
            lmap = self.map[kmer] 
            for char in self.order:
                fout.write(str(lmap[char]) + '\n')
        fout.write('#end\n')        
        fout.close()

    def open(self,filename):
        self.map = {}
        data = open(filename).readlines()
        pos = 1
        while '#' != data[pos][0] :
            kmer = string.strip(data[pos])
            self.map[kmer] = {}
            lmap = self.map[kmer]
            for char in self.order:
                pos = pos + 1
                lmap[char] = int(data[pos])
            pos = pos + 1

    def _is_equiv(self,other):
        for key in self.map.keys():
            for char in self.order:
                assert(self.map[key][char] == other.map[key][char])


    def get_count(self, kmer, char):
        return self.map[kmer][char]

    def make(self):
        for (_kmer,_local_map) in self.map.items():
            _sum = 0
            for _count in _local_map.values():
                _sum = _sum + _count
            for (_char,_count) in _local_map.items():
                self.map[_kmer][_char] = float(_count)/_sum

    def _get_next(self,local_map):
        _random = random.uniform(0.0,1.0)
        _upper_bound = 0
        for (_char,_prob) in local_map.items():
            _upper_bound = _upper_bound + _prob
            if _random <= _upper_bound:
                return _char

    def generate_random_sequence(self,L,kmer):
        ### self.make()

        _buf = []
        for _char in kmer:
            _buf.append(_char)

        while len(_buf) < L:

            # making the last kmer. 
            _last_kmer = []
            for _index in range(-len(kmer),0):
                _last_kmer.append(_buf[_index])
            _last_kmer = string.join(_last_kmer,"")

            _next_char = self._get_next(self.map[_last_kmer])
            _buf.append(_next_char)

        return string.join(_buf,"")


def test():
    seq = ['ATCGGA','GGGCGA','GGCGGC']
    aMarkovModel = MarkovModel()
    aMarkovModel.initialize(3,seq)

    assert(aMarkovModel.get_count('ATC','G') == 1)
    assert(aMarkovModel.get_count('TCG','G') == 1)
    assert(aMarkovModel.get_count('CGG','A') == 1)
    assert(aMarkovModel.get_count('CGG','C') == 1)
    assert(aMarkovModel.get_count('GGG','C') == 1)
    assert(aMarkovModel.get_count('GGC','G') == 2)
    assert(aMarkovModel.get_count('GCG','A') == 1)
    assert(aMarkovModel.get_count('GCG','G') == 1)

    aMarkovModel.make()

    for key,value in aMarkovModel.map.items():
        print key, ' : ', value 

    print 
    print aMarkovModel.generate_random_sequence(10,"ATC")



def test_large_data():
    seq = ['CCACACCACACCCACACACCCACACACCACACCACACACCACACCACACCCACACACACACATCCTAACACTACCCTAACACAGCCCTAATCTAACCCTGGCCAACCTGTCTCTCAACTT']
    aMarkovModel = MarkovModel()
    aMarkovModel.initialize(3,seq)
    aMarkovModel.make()
    print aMarkovModel.generate_random_sequence(1000,"ATC")


def test_large_data_2():
    seq = ['AGTGGGGGAATGAGACAGGGCATGGGGTGGTGAGGTAAGTGCCGTGGATTGTGATGATGGAGAGGGAGGGTAGTTGACATGGAGTTAGAATTGGGTCAGTGTTAGTGTTAGTGTTAGTATTAGGGTGTGGTGTGTGGGTGTGGTGTGGGTGTGGGTGTGGGTGTGGGTGTGGGTGTGGGTGTGGTGTGGTGTGTGGGTGTGGTGTGGGTGTGGTGTGTGTGGG']
    aMarkovModel = MarkovModel()
    aMarkovModel.initialize(3,seq)
    aMarkovModel.make()
    print aMarkovModel.generate_random_sequence(1000,"ATC")


if __name__ == '__main__':
    test_large_data_2()



    
