#!/usr/bin/env python

"""
==============================================================================
SUPERPOSITION : Motif Finder
Authors: Osamu Maruyama (om@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
and
Daisuke Shinozaki (ma202035@math.kyushu-u.ac.jp)
Faculty of Mathematics, Kyushu University
Copyright (c) 2003 Osamu Maruyama and Daisuke Shinozaki
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in
all copies of this software.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR HAS BEEN ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
ON AN "AS IS" BASIS, AND THE AUTHOR HAS NO OBLIGATION TO
PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
==============================================================================
"""

import getopt,sys,time,random
import config,genbank,util,markov,Spp_extract


def options(type):
    
    if type == "short":
        return "P:t:r:s:e:o:g:u:h"
    if type == "long":
        return ['help',    
                'spacer='
                ]


def usage():
    print 
    print 'USAGE'
    print
    print '$ ./superposition.py [options] -o <output file> <positive example file> '
    print
    print      
    print 'OPTIONS'
    print
    print '-P (int)  : The total length of the non-spacer parts of a degenerate pattern (default 6).'
    print '-t (int)  : The number of degenerate patterns to be outputted (default = 100).'
    print '-r (int)  : The size of random sequence set (default = 100).'
    print '-s (int)  : The start position of a upstream region (default -800).'
    print '-e (int)  : The length of a upstrem region (default 800).' 
    print '-u (int)  : The upper bound on the degenerality of a degenerate pattern (default = 6).'     
    print '-h,--help : Show this help message.'

    print '--spacer (int): The length of the central spacer of a degenerate pattern (default = 0).'  
    print 

    
def main():
    """
    Defaults:
    """
    LenLR = 6               # -P
    NumTop = 100            # -t
    NegExNum = 100          # -r
    UpperDgen = 6           # -u
    UpstreamStart = -800    # -s
    UpstreamLength = 800    # -e
    OutputPrefix = None     # -o
    Spacer = None           # --spacer

    try:
        opts, args = getopt.getopt(sys.argv[1:], options("short"), options("long"))
        print 'opts', opts
        print 'args', args
    except getopt.GetoptError:
        usage()
        sys.exit()

    for o, a in opts:
        if o in ['-h', '--help']:
            usage()
            sys.exit(0)

        if o in ['-u']:
            UpperDgen = int(a)
            
        if o in ['--spacer']:
            Spacer = int(a)
        
        if o in ['-P']:
            LenLR = int(a)

        if o in ['-t']:
            NumTop = int(a)
        if o in ['-r']:
            NegExNum = int(a)

        if o in ['-s']:
            UpstreamStart = int(a)
        if o in ['-e']:
            UpstreamLength = int(a)
            
        if o in ['-o']:
            OutputPrefix = a


    PosEx = []
    if args:
        for arg in args:
            PosEx = PosEx + util.ScanFileLineByLine(arg)
    else:
        print('please input file name.')
        sys.exit()
    
    if len(PosEx) == 0:
        sys.stdout.write('\nNo positive examples...\n')
        usage()
        sys.exit()

    try:
        Output= open(OutputPrefix,'w')
    except:
        print '\nGiven me option -o <output file name>.'
        sys.exit()
        
    Output.write('# begin: ' + time.ctime(time.time()) + '\n')
    Output.write("# opts: " + str(opts) + "\n")
    Output.write("# args: " + str(args) + "\n")
    Output.write('\n')
    Output.flush()

           
    GenomeUR = None # not used. 

    print 'Loading GenBank files...' 

    aUR = genbank.UpstreamRegion()
    organism = "S_cerevisiae" # This should be optionized!
    GENOME_filename = 'genome__' + organism + '__' + str(UpstreamStart) + '__' + str(UpstreamLength)
    try:
        aUR.open(config.VAR_DIR + '/' + GENOME_filename)
    except IOError:
        aUR.make(organism,config.DB_DIR,[UpstreamStart,UpstreamStart+UpstreamLength-1])
        aUR.save(config.VAR_DIR + '/' + GENOME_filename)


    PosExSeq = []
    for each in PosEx:
        PosExSeq.append(aUR[each].get_sequence())
    print 'len(PosExSeq) =', len(PosExSeq) 

    k_mer = 3

    print 'Gathering the first', k_mer, 'mer...'
    first_k_mer_list = []
    for each in aUR.get_all_sequences():
        first_k_mer_list.append(each[:k_mer])


    print 'Making Markov table...'
    aMarkovModel = markov.MarkovModel()
    MM_file_full_path = config.VAR_DIR + '/' + 'MM__' + str(UpstreamStart) + '__' + str(UpstreamLength)
    try:
        aMarkovModel.open(MM_file_full_path)
    except IOError:
        aMarkovModel.initialize(k_mer,aUR.get_all_sequences())
        aMarkovModel.save(MM_file_full_path)
    aMarkovModel.make()


    print "Making random sequences..."
    NegExSeq = []

    for Index in range(NegExNum):
        x = random.randrange(len(first_k_mer_list))
        _prefix = first_k_mer_list[x]
        
        Seq = aMarkovModel.generate_random_sequence(UpstreamLength,_prefix)
        NegExSeq.append(Seq)


    ExSeq = {}
    ExSeq['P'] = PosExSeq
    ExSeq['N'] = NegExSeq

    if Spacer <= 0:
        Spacer = None
                
    Spp_extract.main_routine(Output,ExSeq,LenLR,NumTop,UpperDgen,Spacer)

    print 'finish!'
    sys.exit()
        
    
if __name__ == '__main__':
    main()
    print 'out of main'
